"""
CS261 Project 1 - HTML Filter
"""

import sys
import re
from HTMLParser import HTMLParser


allowedTags = ['abbr', 'acronym', 'address', 'area', 'b', 'bdo', 'big', 'blockquote','body', 'br', 'button', 'caption', 'center',
               'cite', 'code', 'col', 'colgroup', 'dd', 'del', 'dfn', 'dir', 'div', 'dl', 'dt', 'em', 'fieldset', 'font', 'h1', 
               'h2', 'h3', 'h4', 'h5', 'h6', 'head', 'hr', 'html', 'i', 'input', 'ins', 'kbd', 'label', 'legend', 'li',
               'map', 'menu', 'noscript', 'ol', 'optgroup', 'option', 'p', 'pre', 'q', 's', 'samp', 'select', 'small', 'span', 'strike',
               'strong', 'sub', 'sup', 'table', 'tbody', 'td', 'textarea', 'tfoot', 'th', 'thead', 'title', 'tr', 'tt', 'u', 'ul', 'var']

specialTags = ['a', 'img', 'form']

allowedAttributes = ['abbr', 'accept', 'accept-charset', 'accesskey', 'align', 'alt', 'axis', 'border', 'cellpadding',
                     'cellspacing', 'char', 'charoff', 'charset', 'checked', 'cite', 'class', 'clear', 'cols', 'colspan', 'color',
                     'compact', 'coords', 'datetime', 'dir', 'disabled', 'enctype', 'for', 'frame', 'headers', 'height',
                     'hreflang', 'hspace', 'id', 'ismap', 'label', 'lang', 'longdesc', 'maxlength', 'media', 'method', 'multiple',
                     'name', 'nohref', 'noshade', 'nowrap', 'prompt', 'readonly', 'rel', 'rev', 'rows', 'rowspan', 'rules', 'scope',
                     'selected', 'shape', 'size', 'span', 'start', 'summary', 'tabindex', 'target', 'title', 'type', 'usemap',
                     'valign', 'value', 'vspace', 'width']

specialAttributes = ['src', 'href'] # 'action'

class MyHTMLParser( HTMLParser ):
    
    unwanted = None
    
    def handleAllowedTag( self, tag, attrs, selfClosing ):
        startTag = "<" + tag
        for attr in attrs:
            if attr[0] in allowedAttributes:
                startTag += " " + attr[0] + '="' + attr[1] +'"'
        if selfClosing: startTag += "/"
        startTag += ">"
        print startTag
    
    def relativeLinkOrNothing( self, attr ):
        start = " " + attr[0] + '="'
        if attr[1].find( '..' ) != -1:
            return start + '"'
        match = re.match( r"/?([a-zA-Z0-9 _\-\.%#]+/?)+" , attr[1] )
        if match and match.string[match.start():match.end()] == attr[1]:
            return start + attr[1] +'"'
        return start + '"'
    
    def handleSpecialTag( self, tag, attrs, selfClosing ):
        startTag = "<"
        if tag == "form":
            startTag += "span"
        else:
            startTag += tag
        
        for attr in attrs:
            if attr[0] in allowedAttributes:
                startTag += " " + attr[0] + '="' + attr[1] +'"'
            elif attr[0] in specialAttributes:
                startTag += self.relativeLinkOrNothing( attr )
        if selfClosing: startTag += "/"
        startTag += ">"
        print startTag
    
    def handleSpecialEndTag( self, tag ):
        if tag == "form":
            print "</span>"
        else:
            print "</" + tag + ">"
    
    def handleStartTags( self, tag, attrs, selfClosing ):
        if self.unwanted == None:
            if tag in allowedTags:
                self.handleAllowedTag( tag, attrs, selfClosing )
            elif tag in specialTags:
                self.handleSpecialTag( tag, attrs, selfClosing )
            elif not selfClosing:
                self.unwanted = ( tag, 1 )
        else:
            if tag == self.unwanted[0]: 
                self.unwanted = ( self.unwanted[0], self.unwanted[1] + 1 )
    
    def handle_starttag( self, tag, attrs ):
        self.handleStartTags( tag, attrs, False )
        
    def handle_endtag( self, tag ):
       if self.unwanted:
           if tag == self.unwanted[0]:
               self.unwanted = ( self.unwanted[0], self.unwanted[1] - 1 )
           if self.unwanted[1] == 0:
               self.unwanted = None
       elif tag in allowedTags:
           print "</%s>" % tag
       elif tag in specialTags:
           self.handleSpecialEndTag( tag )
    
    def handle_startendtag( self, tag, attrs ):
        self.handleStartTags( tag, attrs, True )
    
    def handle_data( self, data ):
        if not self.unwanted:
            print re.sub( r'\n+', lambda x: ' ', data )
   

def sanitizeAndPrint( data ):
    parser = MyHTMLParser()
    parser.feed( data )
    try:
        parser.close()
    except:
        print "Error parsing HTML file, aborting..."

def main( argv ):
    htmlData = sys.stdin.read()
    sanitizeAndPrint( htmlData )
    
    return 0


main( sys.argv );
